;; config-iti.lsp				11 Sep 95
;;------------------------------------------------------------

;;;
;;; Site specific definitions
;;;

(in-package "LISP")
(mapcar #'fmakunbound '(short-site-name long-site-name
                        editor-name edit-file editor-tempfile))

(defun short-site-name () "THD FB 20")
(defun long-site-name () "Institut fr Theoretische Informatik, TH Darmstadt")

;; The name of the default editor:
(defparameter *editor* "xe -q -w")
(defun editor-name () (or (sys::getenv "EDITOR") *editor*))

;; (edit-file file) edits a file.
(defun edit-file (file)
  (open file :direction :probe :if-does-not-exist :create)
  (shell (format nil "~A ~A"
		 (or (system::getenv "VISUAL")
		     (system::getenv "EDITOR")
		     *editor*)
		 (truename file)))
)

;; The temporary file LISP creates for editing:
(defun editor-tempfile ()
  (merge-pathnames "lisptemp.lsp" (user-homedir-pathname))
)

;; The list of directories where programs are searched on LOAD etc.:
(defparameter *load-paths*
  '(#"./" #"../lisplib/" "~/lib/clisp/" #"/usr/local/lib/clisp/"))

;; This makes screen output prettier:
(setq *print-pretty* t)

;; This function will be called right after a clisp session was
;; started. It simply reads the user's rc-file, if it exists.
(defun initialize-lisp-session ()
  ;; for files with a dot in their filename a pathname must be
  ;; constructed explicitly
  (load (merge-pathnames (make-pathname :name ".clisprc") "~/")
	:verbose nil :if-does-not-exist nil))

(defmacro saveinitmem-clisprc (&optional (filename "clisp-nozip.mem")
			       &rest args)
  "Equivalent to 'saveinitmem', but also arranges that the user specific
initialization file '.clisprc' will be loaded each time at startup of
the saved image."
  `(saveinitmem ,filename :init-function #'initialize-lisp-session ,@args))
